import pytest
import os

is_ci = os.environ.get("CI", "").lower() == "true"


@pytest.mark.skipif(is_ci, reason="CI doesn't usually have WiFi")
def test_nm_loc():

    mozloc = pytest.importorskip("mozloc")
    loc = mozloc.parse_signal(mozloc.get_signal())

    assert isinstance(loc, list)
    assert isinstance(loc[0], dict)
    assert -130 < int(loc[0]["signalStrength"]) < 0, "impossible RSSI"


@pytest.mark.skipif(is_ci, reason="CI doesn't usually have WiFi")
def test_nm_connection():

    mozloc = pytest.importorskip("mozloc")
    assert mozloc.cli_config_check()



"""convert logged positions to KML"""

import argparse

from ..utils import csv2kml

p = argparse.ArgumentParser()
p.add_argument("logfn", help="csv logfile to read")
p.add_argument("kmlfn", help="kml filename to write")
p = p.parse_args()

csv2kml(p.logfn, p.kmlfn)



from time import sleep
from pathlib import Path
import logging
from pprint import pprint

from .modules import get_signal, parse_signal, cli_config_check
from .web import get_loc_mozilla

HEADER = "time                lat        lon         accuracy NumBSSIDs"


def process_file(file: Path, mozilla_url: str):
    """
    process raw data captured from NetSH etc. by user previously to a file
    """

    raw = Path(file).expanduser().read_text()
    dat = parse_signal(raw)
    pprint(dat)
    loc = get_loc_mozilla(dat, url=mozilla_url)

    stat = f'{loc["t"].isoformat(timespec="seconds")} {loc["lat"]} {loc["lng"]} {loc["accuracy"]:.1f} {loc["N"]:02d}'

    print(stat)


def log_wifi_loc(cadence_sec: float, mozilla_url: str, logfile: Path = None):

    if logfile:
        logfile = Path(logfile).expanduser()
        with logfile.open("a") as f:
            f.write(HEADER + "\n")

    print(f"updating every {cadence_sec} seconds")
    print(HEADER)

    if not cli_config_check():
        raise ConnectionError("Could not connect to WiFi hardware")
    # nmcli errored for less than about 0.2 sec.
    sleep(0.5)
    while True:
        raw = get_signal()
        dat = parse_signal(raw)
        if len(dat) < 2:
            logging.warning(f"cannot locate since at least 2 BSSIDs required\n{dat}")
            sleep(cadence_sec)
            continue
        logging.debug(dat)

        loc = get_loc_mozilla(dat, mozilla_url)
        if loc is None:
            logging.warning(f"Did not get location from {len(dat)} BSSIDs")
            sleep(cadence_sec)
            continue

        stat = f'{loc["t"].isoformat(timespec="seconds")} {loc["lat"]} {loc["lng"]} {loc["accuracy"]:.1f} {loc["N"]:02d}'
        print(stat)

        if logfile:
            with logfile.open("a") as f:
                f.write(stat + "\n")

        sleep(cadence_sec)



from simplekml import Kml
from pathlib import Path
import pandas


def csv2kml(csvfn: Path, kmlfn: Path):
    """
    convert csv to mkl

    Parameters
    ----------

    csvfn: pathlib.Path
      CSV filename to read
    kmlfn: pathlib.Path
       KML filename to write
    """

    # lon, lat
    dat = pandas.read_csv(csvfn, sep=" ", index_col=0, header=0)

    t = dat.index.tolist()
    lla = dat.loc[:, ["lon", "lat"]].values
    # %% write KML
    """
    http://simplekml.readthedocs.io/en/latest/geometries.html#gxtrack
    https://simplekml.readthedocs.io/en/latest/kml.html#id1
    https://simplekml.readthedocs.io/en/latest/geometries.html#simplekml.GxTrack
    """
    kml = Kml(name="My Kml")
    trk = kml.newgxtrack(name="My Track")
    trk.newwhen(t)  # list of times. MUST be format 2010-05-28T02:02:09Z
    trk.newgxcoord(lla.tolist())  # list of lon,lat,alt, NOT ndarray!

    # just a bunch of points
    #        for i,p in enumerate(lla): # iterate over rows
    #            kml.newpoint(name=str(i), coords=[p])

    print("writing", kmlfn)
    kml.save(kmlfn)



import requests
import logging
import pandas
import json
import typing as T
from datetime import datetime


def get_loc_mozilla(dat: T.Sequence[T.Any], url: str):

    if isinstance(dat, pandas.DataFrame):
        json_to = dat.to_json(orient="records")
    elif isinstance(dat, list):
        json_to = json.dumps(dat)
    else:
        raise TypeError("Unknown data format")

    json_to = '{ "wifiAccessPoints":' + json_to + "}"
    try:
        req = requests.post(url, data=json_to)
        if req.status_code != 200:
            logging.error(req.text)
            return None
    except requests.exceptions.ConnectionError as e:
        logging.error(f"no network connection.  {e}")
        return None
    # %% process MLS response
    json_from = req.json()
    loc = json_from["location"]
    loc["accuracy"] = json_from["accuracy"]
    loc["N"] = len(dat)  # number of BSSIDs used
    loc["t"] = datetime.now()

    return loc



import sys

if sys.platform == "win32":
    from .netsh import cli_config_check, get_signal, parse_signal
elif sys.platform == "linux":
    from .netman import cli_config_check, get_signal, parse_signal
elif sys.platform == "darwin":
    from .airport import cli_config_check, get_signal, parse_signal
else:
    raise ImportError(f"MozLoc doesn't work with platform {sys.platform}")



""" MacOS airport functions """

from __future__ import annotations
import typing as T
import shutil
import logging
import subprocess
import re

EXE = shutil.which("airport", path="/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources")
if not EXE:
    raise ImportError("Could not find Airport")


def cli_config_check() -> bool:
    # %% check that Airport is available and WiFi is active
    ret = subprocess.run([EXE, "--getinfo"], stdout=subprocess.PIPE, text=True, timeout=30)

    if ret.returncode != 0:
        return False

    stdout = ret.stdout.strip().lower()
    if len(stdout) == "airport: off":
        return False

    if "state: running" in stdout:
        return True

    logging.error("could not determine WiFi state.")
    return False


def get_signal() -> str:

    ret = subprocess.run([EXE, "-s"], timeout=30.0, stdout=subprocess.PIPE, text=True)

    if ret.returncode != 0:
        logging.error("consider slowing scan cadence.")

    return ret.stdout


def parse_signal(raw: str) -> list[dict[str, T.Any]]:

    pat = re.compile(r"\s*([0-9a-zA-Z\-\.]+)\s+([0-9a-f]{2}(?::[0-9a-f]{2}){5})\s+(-\d{2,3})")
    dat: list[dict[str, str]] = []

    for line in raw.split("\n"):
        mat = pat.match(line)
        if mat:
            ssid = mat.group(1)
            # optout
            if ssid.endswith("_nomap"):
                continue

            dat.append({"ssid": ssid, "macAddress": mat.group(2), "signalStrength": mat.group(3)})

    return dat



"""
https://mozilla.github.io/ichnaea/api/geolocate.html
https://ichnaea.readthedocs.io/en/latest/api/geolocate.html

you should get your own Mozilla Location Services API key

Don't abuse the API or you'll get banned (excessive polling rate)
"""

import argparse

from .base import log_wifi_loc, process_file


p = argparse.ArgumentParser()
p.add_argument("logfile", help="logfile to append location to", nargs="?")
p.add_argument(
    "-T", "--cadence", help="how often to update [sec]. Some laptops cannot go faster than 30 sec.", default=60, type=float
)
p.add_argument(
    "-url",
    help="Mozilla location services URL--don't use this default test key",
    default="https://location.services.mozilla.com/v1/geolocate?key=test",
)
p.add_argument("-i", "--infile", help="use raw text saved from command line")
p = p.parse_args()

if p.infile:
    process_file(p.infile, mozilla_url=p.url)
else:
    log_wifi_loc(cadence_sec=p.cadence, mozilla_url=p.url, logfile=p.logfile)
